Package: qtl2
Version: 0.44
Date: 2026-07-06
Title: Quantitative Trait Locus Mapping in Experimental Crosses
Description: Provides a set of tools to perform quantitative
    trait locus (QTL) analysis in experimental crosses. It is a
    reimplementation of the 'R/qtl' package to better handle
    high-dimensional data and complex cross designs.
    Broman et al. (2019) <doi:10.1534/genetics.118.301595>.
Author: Karl W Broman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4914-6671>),
  R Core Team [ctb]
Maintainer: Karl W Broman <broman@wisc.edu>
Authors@R: c(person("Karl W", "Broman", role=c("aut", "cre"),
             email="broman@wisc.edu", comment=c(ORCID = "0000-0002-4914-6671")),
             person("R Core Team", role="ctb"))
Copyright: Code for Brent's method for univariate function optimization
        was taken from R 3.2.2 (Copyright 1995, 1996 Robert Gentleman
        and Ross Ihaka, Copyright 2003-2004 The R Foundation, Copyright
        1998-2014 The R Core Team).
Depends: R (>= 3.1.0)
Imports: Rcpp (>= 1.0.7), yaml (>= 2.1.13), jsonlite (>= 0.9.17),
        data.table (>= 1.10.4-3), parallel, stats, utils, graphics,
        grDevices, RSQLite
Suggests: testthat, devtools, roxygen2, vdiffr, qtl
License: GPL-3
URL: https://kbroman.org/qtl2/, https://github.com/rqtl/qtl2
BugReports: https://github.com/rqtl/qtl2/issues
LazyData: true
Encoding: UTF-8
ByteCompile: true
LinkingTo: Rcpp, RcppEigen
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-07-06 12:47:21 UTC; kbroman
Repository: CRAN
Date/Publication: 2026-07-06 14:10:07 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-07-06 15:36:27 UTC; unix
Archs: qtl2.so.dSYM
