# Generated by roxygen2: do not edit by hand

S3method("[",calc_genoprob)
S3method("[",cross2)
S3method("[",phasedgeno)
S3method("[",sim_geno)
S3method("[",viterbi)
S3method(c,scan1perm)
S3method(cbind,calc_genoprob)
S3method(cbind,phasedgeno)
S3method(cbind,scan1)
S3method(cbind,scan1perm)
S3method(cbind,sim_geno)
S3method(cbind,viterbi)
S3method(clean,calc_genoprob)
S3method(clean,scan1)
S3method(dim,calc_genoprob)
S3method(dimnames,calc_genoprob)
S3method(max,compare_geno)
S3method(max,scan1)
S3method(plot,calc_genoprob)
S3method(plot,compare_geno)
S3method(plot,scan1)
S3method(plot,scan1coef)
S3method(print,cross2)
S3method(print,summary.compare_geno)
S3method(print,summary.cross2)
S3method(print,summary.scan1perm)
S3method(rbind,calc_genoprob)
S3method(rbind,phasedgeno)
S3method(rbind,scan1)
S3method(rbind,scan1perm)
S3method(rbind,sim_geno)
S3method(rbind,viterbi)
S3method(replace_ids,calc_genoprob)
S3method(replace_ids,cross2)
S3method(replace_ids,data.frame)
S3method(replace_ids,matrix)
S3method(replace_ids,sim_geno)
S3method(replace_ids,viterbi)
S3method(subset,calc_genoprob)
S3method(subset,cross2)
S3method(subset,phasedgeno)
S3method(subset,scan1)
S3method(subset,sim_geno)
S3method(subset,viterbi)
S3method(summary,compare_geno)
S3method(summary,cross2)
S3method(summary,scan1perm)
export(add_threshold)
export(align_scan1_map)
export(batch_cols)
export(batch_vec)
export(bayes_int)
export(calc_entropy)
export(calc_errorlod)
export(calc_geno_freq)
export(calc_genoprob)
export(calc_grid)
export(calc_het)
export(calc_hotspots)
export(calc_kinship)
export(calc_raw_founder_maf)
export(calc_raw_geno_freq)
export(calc_raw_het)
export(calc_raw_maf)
export(calc_sdp)
export(cbind_expand)
export(check_cross2)
export(chisq_colpairs)
export(chr_lengths)
export(chr_names)
export(clean)
export(clean_genoprob)
export(clean_scan1)
export(compare_founder_geno)
export(compare_geno)
export(compare_genoprob)
export(compare_maps)
export(convert2cross2)
export(count_xo)
export(covar_names)
export(create_gene_query_func)
export(create_snpinfo)
export(create_variant_query_func)
export(decomp_kinship)
export(drop_markers)
export(drop_nullmarkers)
export(est_herit)
export(est_map)
export(find_dup_markers)
export(find_ibd_segments)
export(find_index_snp)
export(find_map_gaps)
export(find_marker)
export(find_markerpos)
export(find_peaks)
export(fit1)
export(founders)
export(fread_csv)
export(fread_csv_numer)
export(genoprob_to_alleleprob)
export(genoprob_to_snpprob)
export(get_common_ids)
export(get_x_covar)
export(guess_phase)
export(ind_ids)
export(ind_ids_covar)
export(ind_ids_geno)
export(ind_ids_gnp)
export(ind_ids_pheno)
export(index_snps)
export(insert_pseudomarkers)
export(interp_genoprob)
export(interp_map)
export(invert_sdp)
export(locate_xo)
export(lod_int)
export(map_to_grid)
export(marker_names)
export(mat2strata)
export(max_compare_geno)
export(max_scan1)
export(maxlod)
export(maxmarg)
export(n_chr)
export(n_covar)
export(n_founders)
export(n_ind)
export(n_ind_covar)
export(n_ind_geno)
export(n_ind_gnp)
export(n_ind_pheno)
export(n_mar)
export(n_missing)
export(n_pheno)
export(n_phenocovar)
export(n_typed)
export(pheno_names)
export(phenocovar_names)
export(plot_ci)
export(plot_cistrans)
export(plot_coef)
export(plot_coefCC)
export(plot_colorscale)
export(plot_compare_geno)
export(plot_genes)
export(plot_geno)
export(plot_genoprob)
export(plot_genoprobcomp)
export(plot_lodpeaks)
export(plot_onegeno)
export(plot_peaks)
export(plot_pxg)
export(plot_scan1)
export(plot_scan1_heatmap)
export(plot_sdp)
export(plot_snpasso)
export(predict_snpgeno)
export(probs_to_grid)
export(pull_genoprobint)
export(pull_genoprobpos)
export(pull_markers)
export(qtl2version)
export(read_cross2)
export(read_pheno)
export(recode_snps)
export(reduce_map_gaps)
export(reduce_markers)
export(replace_ids)
export(scale_kinship)
export(scan1)
export(scan1blup)
export(scan1coef)
export(scan1gen)
export(scan1max)
export(scan1perm)
export(scan1snps)
export(sdp2char)
export(sim_geno)
export(smooth_gmap)
export(subset_scan1)
export(summary_compare_geno)
export(summary_scan1perm)
export(top_snps)
export(tot_mar)
export(unsmooth_gmap)
export(viterbi)
export(write_control_file)
export(xpos_scan1)
export(zip_datafiles)
importFrom(RSQLite,SQLite)
importFrom(RSQLite,dbConnect)
importFrom(RSQLite,dbDisconnect)
importFrom(RSQLite,dbGetQuery)
importFrom(Rcpp,sourceCpp)
importFrom(data.table,fread)
importFrom(grDevices,gray)
importFrom(grDevices,heat.colors)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,terrain.colors)
importFrom(grDevices,topo.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(parallel,detectCores)
importFrom(stats,complete.cases)
importFrom(stats,lm)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(utils,packageVersion)
useDynLib(qtl2, .registration=TRUE)
